@@ -11,9 +11,8 @@ import platform
 
 from fmqlutils import VISTA_DATA_BASE_DIR
 from fmqlutils.reporter.reportUtils import MarkdownTable, reportAbsAndPercent, muBVC
-from fmqlreports.webReportUtils import reduce2
+from fmqlreports.webReportUtils import reduce2, vistasOfVISNByOne
 
-from fileDirectory import VISN20_VISTAS # TMP - move to webReportUtils
 from userDirectory import CommonUserDirectory
 
 """
@@ -296,7 +295,13 @@ class CommonPatientDirectory():
     TODO: could move to simpler, get name form from firstEntry as ALL ENTRIES seem
     to have same name (unlike User). Must re-inforce in Patient Report First.
     """
-    def subsetNDEID(self, patientGIENs, idMaker, nameMaker):
+    def subsetNDEID(self, patientGIENs, idMaker, nameMaker, preGeneratedNames={}):
+        if len(preGeneratedNames):
+            fnsExcluded = set(
+               "{} {}".format(nm.split(" ")[0], nm.split(" ")[-1])
+                for nm in preGeneratedNames.values()
+            )
+            nameMaker.excludeNames(fnsExcluded)
         npatients = []
         nentries = {}
         snos = set()
@@ -310,15 +315,27 @@ class CommonPatientDirectory():
             fentry = self.__pdirInfo["entries"][patient[0]]
             sexes = set([self.__pdirInfo["entries"][gien][2] for gien in patient if self.__pdirInfo["entries"][gien][2]])
             if len(sexes) > 1:
-                raise Exception("Can't de id with SEX as > 1 Sex for a patient")
+                # Some MEANS TEST, TEST PATIENTs are F in one, M in another
+                if re.search(r'PATIENT', fentry[1]):
+                    sexes = set(["M"])
+                else:
+                    raise Exception("Can't de id with SEX as > 1 Sex for a patient: {} - {}".format(json.dumps(patient), json.dumps(fentry)))
             if len(sexes) == 0:
                 sex = "female" if randint(1, 5) == 1 else "male" # 20% female
                 print("** sexless patient: {} - picking {} (Female 20%)".format(self.__pdirInfo["entries"][patient[0]][1], sex))
             else:
                 sex = "male" if list(sexes)[0] == "M" else "female"
             keepOrigName = True if re.search(r'(PATIENT|TEST|\,VET)', fentry[1]) else False
-            if not keepOrigName:    
-                sname, iname, fname = nameMaker.getPatientNames(gender=sex, originalName=fentry[1]) 
+            if not keepOrigName:   
+                nms = []
+                if len(preGeneratedNames): # insist on full: first middle last
+                    for patientGIEN in patient:
+                        if patientGIEN in preGeneratedNames:
+                            nms = preGeneratedNames[patientGIEN].split(" ")
+                            break
+                if len(nms) == 0:
+                    nms = nameMaker.getNames(gender=sex, originalName=fentry[1])
+                sname, iname, fname = nameMaker.calculatePatientNameVariations(nms)
                 namesUsed = set() # see if Patient, like User varies - doesn't seem to
             else:
                 print("** Keeping Original Patient Name {}".format(fentry[1]))
@@ -361,7 +378,7 @@ class CommonPatientDirectory():
                     print("** Putting back ZZ on patient name: {} - {}".format(origName, entry[1]))
             # MAY USE THIS TO SIMPLIFY THE ABOVE ie/ firstEntry good enough for name calc
             if len(namesUsed) > 1:
-                raise Exception("Enforcing - though not relying on - that Patient has same name across entries")
+                print("** Enforcing - though not relying on - that Patient has same name across entries - {}".format(json.dumps(list(namesUsed))))
         print("Returning {:,} entries and {:,} patients from Patient Directory{}".format(len(nentries), len(npatients), " [DEIDENTIFIED]" if nameMaker else ""))
         nvistas = [vinfo for vinfo in self.__pdirInfo["vistas"] if vinfo["stationNumber"] in snos]
         return {"vistas": nvistas, "entries": nentries, "patients": npatients}
@@ -626,7 +643,7 @@ class CommonPatientDirectory():
                 patientInfoByIEN = reduce2(stationNo)    
             except:
                 print("Can't load Patient Reduction for {} as not available/creatable here".format(stationNo))
-                raise
+                continue
             changed = True
             print("Loaded Patients for {}: {:,}".format(stationNo, len(patientInfoByIEN)))
             print("Adding {} to directory".format(stationNo))
@@ -748,12 +765,13 @@ def main():
 
     assert sys.version_info >= (3, 6)
     
-    cud = CommonUserDirectory(VISN20_VISTAS)    
-    cpd = CommonPatientDirectory(VISN20_VISTAS, cud)
+    visn20VistAs = vistasOfVISNByOne("20")
+    cud = CommonUserDirectory(visn20VistAs)    
+    cpd = CommonPatientDirectory(visn20VistAs, cud)
     print(cpd.report())
     json.dump(cpd.entryDatesBySNO(), open("Meta/Common/patientDatesBySNO.json", "w"), indent=4)
     # cpd.dumpSameAsUser()
             
 if __name__ == "__main__":
     main()  
-    
\ No newline at end of file
+    
