@@ -14,21 +14,38 @@ from fmqlutils.cacher.cacherUtils import FMQLReplyStore, FilteredResultIterator,
 from fmqlutils.reporter.reportUtils import MarkdownTable, reportPercent, reportAbsAndPercent, muBVC
 from fmqlutils.typer.reduceTypeUtils import splitTypeDatas, checkDataPresent, singleValue, combineSubTypes, muBVCOfSTProp, refsOfST
 
-from fmqlreports.webReportUtils import TOP_MD_TEMPL, SITE_DIR_TEMPL, ensureWebReportLocations, keyStats, flattenFrequencyDistribution, roundFloat, reduce4, flattenPropValues 
+from fmqlreports.webReportUtils import TOP_MD_TEMPL, SITE_DIR_TEMPL, ensureWebReportLocations, keyStats, flattenFrequencyDistribution, roundFloat, reduce4, flattenPropValues, vistasOfVISNByOne
+from fmqlreports.webReportUtils import muPlotRef, makePlots 
 
 """
-NEXT ON CONTENT (not on visuals):
+NEXT ON CONTENT (+ then back to visuals):
+
+fill in a LOCAL TIU in 123? http://localhost:9100/schema#123_03 (associated results) vs http://localhost:9100/schema#123_051 (remote results) … is this the one leading to RPC.
+<------ need TIU to grab back vs HL7 ... see if 123/remote results never has associated_results and vica versa
+
+KEY TODO (from CPRS manual/consum.pdf for all Consult States:
+- Receive – When the request arrives at the specified service, the designated person, such as a clerk, can use Receive to change the consult status to active. 
+... P67 of the consum.pdf manual
+- Schedule – The service can schedule a consult, which makes the consult available to be resulted.
+- Complete Request also links you to TIU so that you can enter findings.
 
 - Filler breakdown ala Placer (no mixing for now)
 - may only do completed ie/ nix discontinued/cancelled for now
 - expose remote site service name for placers? ie/ to link em up?
     ifc_remote_service_name (pair with local services so see if 1 remote for > 1 local?)
-- visn named (not just other visns)
+- visn named (not just other visns) <---- now have VISN MAP!!!!!!
 - filler only patients (delay reg?) ... what?
 - WILL START with COMPLETED vs OTHER for all CONSULTs and then just do COMPLETED in rest of report.
 - locations:
     from/... location 44 (no pattern seen per see but do they all have similar workloads?)
     ordering_facility (is precise to clinic)
+- ON VISUALS: TIME SERIES THE IFCs by TYPE per week ... ie/ do time series view of IFCs (+ bring out time series raw numbers too ie/ median/min/max ... SHOW BETTER) <--- ** as can do IFCs for last five years etc)
+
+BIGGIES AFTER: "IFC as organizing area to draw in others" (directories, HL7, appointments etc) 
+- will draw in HL7 too 
+- full draw in of documents
+- ANY appts scheduled? Patient just for IFC or more etc etc
+ie/ "The Travelling Veteran" ... "The Portable Veteran"
 
 Portland vs Pug as big (can do TeleMental as part of filler breaks):
 ... TODO:
@@ -82,7 +99,6 @@ TODO NEXT: for basic, do filler
 - more on if Consult has images or not (teler beyond on image bearing consults?)
 - user and patient discrepancies ... a/c for them <---- make top bars relevant
 - OTHER DATA: TIU, Image, Appts - any connection to consults (IFCs) ie/ anything else made? [will tie to images and encounters] ... also want to see if any have img or doc assoc? [123 has TIU link! and assoc results ie/ PROCESSED IN TELEREADING (do this in there first ie/ any image etc ref from doc?]
-- VISN 20 dynamic! ie/ lookup for any system
 - clean TODO PASS 2 below
 
 USE AS EXCUSE FOR:
@@ -137,50 +153,39 @@ def webReportIFC(stationNo, stationName, onAndAfterDay, upToDay, imageDir="Image
                 
     institByIEN = reduce4(stationNo)
             
+    """
+    Services usually always have/lead to Routing Facility or don't. Want to catch the 
+    "changers" which for now (TODO?), put into WORF by default
+    """
+    wrfServices = set()
+    worfServices = set()
+    for st in sts123YR1:
+        if "routing_facility" in st:
+            wrfServices.add(singleValue(st, "to_service"))
+            continue
+        worfServices.add(singleValue(st, "to_service"))
+    wrfServices = wrfServices - worfServices # exclusive WRFs only
+    
     # WRF - with routing facility => IFC ie/ wrf <=> ifc
-    stWRF = [st for st in sts123YR1 if "routing_facility" in st]
-    placerServicesCheck = set()
+    stWRF = [st for st in sts123YR1 if singleValue(st, "to_service") in wrfServices]
     # If routing_facility => ifc_role and vica_versa
     if sum(1 for st in stWRF if "ifc_role" not in st and "routing_facility" in st) or sum(1 for st in sts123YR1 if "ifc_role" in st and "routing_facility" not in st):
         raise Exception("Where routing facility expect role and where role, expect routing facility")
-    try: # either placer or filler within st gathered by service, routing_facility
-        for st in stWRF:
+    for st in stWRF:
+        try:
             ifcRole = singleValue(st, "ifc_role")
-            # bonus - record placer service for below
-            if ifcRole == "P:PLACER":
-                placerServicesCheck.add(singleValue(st, "to_service"))
-    except:
-        print(json.dumps(st, indent=4))
-        raise Exception("> 1 ifc_role in routing_facility'ed ST. Unexpected")
+        except:
+            print(json.dumps(st, indent=4))
+            raise Exception("> 1 ifc_role in routing_facility'ed ST. Unexpected")       
+    
     byServiceWRF = defaultdict(list)
     for st in stWRF: # note: for filler/local overlaps will only gather the filler st's
         service = singleValue(st, "to_service")
         byServiceWRF[service].append(st)
 
-    # Service overlap with RF may happen due to filler being used in RF 
-    # and WORF consults
-    #
-    # REVISIT: handles case where Placer service DOES not get reused in WORF
-    # (enforced below) BUT Portland has DISCONTINUED case where this happens.
-    # ... perhaps discontinued happens before IFC info added to a Consult? 
-    # ... TODO: follow up as not counting this now!
-    # 
-    stWORF = [st for st in sts123YR1 if not ("routing_facility" in st or singleValue(st, "to_service") in placerServicesCheck)]  
-    # Just for tracing -- the WORF Placers
-    # ... PENDING is fine (but rare); DISCONTINUED seems to be too and then WCO has
-    # an Active that has no further action beyond CPRS submit and just sits there.
-    stWORFPlacer = [st for st in sts123YR1 if "routing_facility" not in st and singleValue(st, "to_service") in placerServicesCheck]
-    if len(stWORFPlacer):
-        for st in stWORFPlacer: # POR (DISCONTINUED), PUG (PENDING)
-            stat = singleValue(st, "cprs_status")
-            if not re.search(r'(ACTIVE|DISCONTINUED|PENDING)', stat):
-                print(json.dumps(st, indent=4))
-                raise Exception("If a Placer Service ST lacks Route Info, expect it to be (quickly?) discontinued or to be pending")
-        print("** Warning: {:,} consults were routeless to Placer Services.".format(
-            sum(st["_total"] for st in stWORFPlacer)
-        ))
+    stWORF = [st for st in sts123YR1 if singleValue(st, "to_service") in worfServices]
+
     worfTTL = sum(st["_total"] for st in stWORF)
-    worfServices = set(singleValue(st, "to_service") for st in stWORF)
     worfUsers = set(user for st in stWORF for user in refsOfST(st, "200"))
     worfPatients = set(patient for st in stWORF for patient in refsOfST(st, "2"))
     
@@ -240,21 +245,10 @@ def webReportIFC(stationNo, stationName, onAndAfterDay, upToDay, imageDir="Image
             placerPatients |= servicePatients
             placerTTL += serviceTTL
             if len(set(rf for st in byServiceWRF[service] for rf in st["routing_facility"]["byValueCount"])) > 1:
-                # ie/ Service --> Place Routed to if Placer. Filler may take from
-                # multiple places.
                 raise Exception("Expected one and only one RF for Placer Services")
                 
-    if placerServices != placerServicesCheck:
-        print(len(placerServices))
-        print(len(placerServicesCheck))
-        raise Exception("Placers analyzed before to avoid WORF problems above - make sure in Sync") # TMP til tackle the DISCONTINUED
     if (placerTTL + fillerTTL + worfTTL) != allTTL:
         raise Exception("Totalling mistake - consult totals are exclusive")
-    # ONLY for no overlap placer and WORF ... filler may be reused so there can be
-    # overlap (ie/ remote VistA reuses a locally used consult service)
-    if len(worfServices.intersection(placerServices)):
-        print(worfServices.intersection(set(placerServices)))
-        raise Exception("Expect Placer to be exclusive from Local Services")
     fillerAndLocalServices = worfServices.intersection(fillerServices)
     if len(allServices) != len(placerServices) + (len(fillerServices) - len(fillerAndLocalServices)) + len(worfServices): # overlap only counted in worfServices
         raise Exception("Expected overlap of filler and local to be easy to a/c for in allTotal") 
@@ -432,9 +426,10 @@ As for users, most tend to only local consults or to a mix of local and filled c
     )
     
     # ################################ Placer ######################
-            
-    # {"ANCHORAGE VA MEDICAL CENTER [363]", "BOISE VA MEDICAL CENTER [531]", "MANN-GRANDSTAFF VAMC [668]", "PORTLAND VA MEDICAL CENTER [648]", "ROSEBURG VA MEDICAL CENTER [653]", "SEATTLE VA MEDICAL CENTER [663]", "WHITE CITY VA MEDICAL CENTER [692]"}
-    VISN20s = {"363": "ANC", "531": "BOI", "687": "WWW", "668": "SPO", "648": "POR", "653": "ROS", "663": "PUG", "692": "WCO"}
+    
+    HOMEVISNVISTAS = vistasOfVISNByOne(stationNo)     
+    print(f"Home VISN VistAs are {HOMEVISNVISTAS}")
+    
     placerServicesFacilities = Counter()
     telerTBL = MarkdownTable([":Placer Service", ":To Facility", "\#", "Status'"])
     placerServicesTeleR = set()
@@ -495,7 +490,8 @@ utomatically generated note - signature not required.\n \n  Electronically Filed
                 dad.dumpDocsOfST(groupLabel, st)
         serviceName = re.sub(r' +$', '', service.split(" [")[0]) 
         facMU = re.sub(r'4\-', '', rf)
-        facMU = f'__{facMU}__' if re.search(r'\-(\d+)\]', rf).group(1) in VISN20s else facMU 
+        vistaSNO = re.search(r'\-(\d+)\]', rf).group(1)
+        facMU = f'__{facMU}__' if re.search(r'\-(\d+)\]', rf).group(1) in HOMEVISNVISTAS else facMU 
         tblToUse.addRow([
             # "__{}__".format(serviceName) if tttl/placerTTL > 0.1 else serviceName,
             serviceName,
@@ -527,13 +523,13 @@ utomatically generated note - signature not required.\n \n  Electronically Filed
         otherTTL = 0
         for facility in sorted(facilityCount, key=lambda x: facilityCount[x], reverse=True):
             facSNO = re.search(r'\-(\d+)\]', facility).group(1)
-            if facSNO in VISN20s:
-                visnValues.append((facilityCount[facility], VISN20s[facSNO]))
+            if facSNO in HOMEVISNVISTAS:
+                visnValues.append((facilityCount[facility], HOMEVISNVISTAS[facSNO]))
             else:
                 otherTTL += facilityCount[facility]
             facMU = re.sub(r'4\-', '', facility)
             # serviceNameMU = "__{}__".format(serviceName) if facilityCount[facility]/ssttl > 0.01 else serviceName
-            facMU = f'__{facMU}__' if facSNO in VISN20s else facMU 
+            facMU = f'__{facMU}__' if facSNO in HOMEVISNVISTAS else facMU 
             if cols[1] == ":Service":
                 row = [
                     facMU,
@@ -632,7 +628,7 @@ Note that equivalent services are filled by this VistA. Those are detailed in th
     if len(placerServicesSpclzed):
         ftbl, facilities, tttlSpclzed, plotRef = summarizePlacerFacilities(placerServicesSpclzed, "Specialized")        
         # TODO: add later -- Placer VistA has many (relatively); a Filler has few - this is reversed in the Filler section    
-        pbdmu += "The balance, <span class='yellowIt'>{}</span>, of the Placer Services to <span class='yellowIt'>{:,}</span> facilities, are mainly for specialized clinical services and represent <span class='yellowIt'>{}</span> of placed consults. Once again, the vast majority go to other VISN 20 facilities but the flow is mainly from smaller facilities to larger with the small placing with the large.\n\n".format(
+        pbdmu += "The balance, <span class='yellowIt'>{}</span>, of the Placer Services to <span class='yellowIt'>{:,}</span> facilities, are mainly for specialized clinical services and represent <span class='yellowIt'>{}</span> of placed consults. Once again, the vast majority go to other facilities in the same VISN but the flow is mainly from smaller facilities to larger with the small placing with the large.\n\n".format(
             len(placerServicesSpclzed), 
             len(facilities), 
             reportPercent(tttlSpclzed, placerTTL)
@@ -676,7 +672,7 @@ Note that equivalent services are filled by this VistA. Those are detailed in th
     
     # May break out 'VISN 20 CANCER CARE NAVIGATION TEAM IFC' -- another common, peer to
     # peer service (all do placer and filler)
-    blurb = "There are <span class='yellowIt'>{:,}</span> __Placer Services__ - services that route/place consults __to__ <span class='yellowIt'>{:,}</span> other facilities. In VistA, a Placer Service identifies a specific remote (routing) facility which is always the main, default facility of a remote VistA. In this VistA, <span class='yellowIt'>{}</span> of the IFC Consults are \"Placed\" and the vast majority go __to other VISN 20 facilities__, facilities highlighted in bold in the tables below. Placer consults can be broken into four broad classes ...\n\n".format(
+    blurb = "There are <span class='yellowIt'>{:,}</span> __Placer Services__ - services that route/place consults __to__ <span class='yellowIt'>{:,}</span> other facilities. In VistA, a Placer Service identifies a specific remote (routing) facility which is always the main, default facility of a remote VistA. In this VistA, <span class='yellowIt'>{}</span> of the IFC Consults are \"Placed\" and the vast majority go __to other facilities in the same VISN__, facilities highlighted in bold in the tables below. Placer consults can be broken into four broad classes ...\n\n".format(
         len(placerServices), 
         len(placerServicesFacilities), 
         reportPercent(placerTTL, wrfTTL)
@@ -712,7 +708,7 @@ Note that equivalent services are filled by this VistA. Those are detailed in th
         # TODO: from facilities don't appear to be ordered
         tbl.addRow([
             serviceName,
-            ", ".join(["{} [{:,}]".format(rf.split(" [")[0] if re.search(r'\[(\d+)\]', rf).group(1) not in VISN20s else f'__{rf.split(" [")[0]}__', rfcntr[rf]) for rf in rfcntr]),
+            ", ".join(["{} [{:,}]".format(rf.split(" [")[0] if re.search(r'\[(\d+)\]', rf).group(1) not in HOMEVISNVISTAS else f'__{rf.split(" [")[0]}__', rfcntr[rf]) for rf in rfcntr]),
             reportAbsAndPercent(tttl, fillerTTL),
             muBVC(scntr)
         ])
@@ -729,85 +725,6 @@ Note that equivalent services are filled by this VistA. Those are detailed in th
     userSiteDir = SITE_DIR_TEMPL.format(stationNo)
     print(f'Serializing Report to {userSiteDir}')
     open(f'{userSiteDir}ifcs.md', "w").write(mu) 
-
-# ##################### Plot Creation #########################
-        
-"""
-TMP -- will move to webReportUtils when happy
-
-Plot creation is separate --- may move into Visualize so common
-
-ex/ plotDataFile = "plotDataIFC.json" ie/ plotDataFileSuffix="IFC"
-
-TODO:
-- ifcPlTravelVISNSankey (from Travel PlacersVISNSankey) -- .md change too
-- all consult services categ ... -> ifcAllConsultsCategory 
-- remove 0 ie/ NO OTHER!
-- map SEA -> PUG, MAN -> SPO
-
-"""
-
-def makePlots(stationNo, plotDataFileSuffix):   
-
-    import io
-    import pandas as pd
-
-    from visualize import Visualize
-    imageDir = f'{VISTA_DATA_BASE_DIR}{stationNo}/ReportSite/Images'
-    if not os.path.isdir(imageDir):
-        os.mkdir(imageDir)
-    viz = Visualize(imageDir)
-    plotData =  json.load(open(f'{VISTA_DATA_BASE_DIR}{stationNo}/TmpWorking/plotData{plotDataFileSuffix}.json'))
-    for plotDataName in plotData:
-        plotDataInfo = plotData[plotDataName]
-        print(f'Remaking {plotDataInfo["title"]}')
-        method = getattr(viz, plotDataInfo["plotMethod"])
-        if "rows" in plotDataInfo:
-            arg = viz.makeDF(
-                data=plotDataInfo["data"],
-                columns=plotDataInfo["columns"],
-                rows=plotDataInfo["rows"]
-            )
-        elif "specs" in plotDataInfo:
-            arg = plotDataInfo["specs"]
-        elif "dfcsv" in plotDataInfo:
-            dfcsv = plotDataInfo["dfcsv"]
-            df = pd.read_csv(
-                io.StringIO("\n".join(dfcsv)),
-                index_col=0, 
-                parse_dates=True
-            )
-            arg = df
-        else:
-            raise Exception("Expect plot data for DF or as 'specs'")
-        kargs = plotDataInfo.get("kargs", {})
-        plotRef = method(arg, plotDataInfo["title"], plotDataInfo["plotName"], **kargs)
-        print(f'\texpect reference to {plotRef} in report')
-        
-def muPlotRef(plotInfo, flushToDirectory):
-    """
-    # Want separate from visualize for now BUT possible break
-    # as separate from Visualize naming impl
-    
-    ex/ ![ifcTravelVISNSankey](Images/ifcTravelVISNSankeyBGW.svg)
-    """
-    def svgPlotFile(plotName, plotMethodName, plotKArgs, flushToDirectory):
-        """
-        So don't have to make the plot but can know its name as making md
-        Compatible with Visualize.__flushPlot's naming of SVG's
-        """
-        coreMethodName = re.sub("plot", "", plotMethodName)
-        coreMethodNameU = coreMethodName[0].upper() + coreMethodName[1:]
-        plotFilePrefix = f'{plotName}{coreMethodNameU}'
-        if plotKArgs:
-            if "start" in plotKArgs:
-                plotFilePrefix += plotKArgs["start"]
-            if "end" in plotKArgs:
-                plotFilePrefix += "-{}".format(plotKArgs["end"])
-        plotFile = f'{flushToDirectory}/{plotFilePrefix}BGW.svg'
-        return plotFile, plotFilePrefix
-    plotFile, plotFilePrefix = svgPlotFile(plotInfo["plotName"], plotInfo["plotMethod"], plotInfo["kargs"] if "kargs" in plotInfo else None, flushToDirectory)
-    return "![{}]({})".format(plotFilePrefix, plotFile)
     
 """
 TMP HERE - move to IFCRawDataAnalysis.py for raw data analysis of IFC pieces
