@@ -32,7 +32,7 @@ Requires:
 TODO: see other xxrepo notes <----- nix when through
 
 TODO v1.1:
-- as doing: introduce singleValue, singleValueCount on prop which enforces singletons ie/ reduces need for .keys()
+- as doing: introduce singleValue, singleValueCount on prop which enforces singletons 
 - assigned are mixed with types in table and not in old reports => split out and fill in assignements ala report
 - tie to docs: 8925 ... 26_14
 
@@ -59,7 +59,7 @@ def webReportPRFs(stationNo):
                 
     all26_13Data, st26_13s = splitTypeDatas(stationNo, "26_13", expectSubTypeProperty="status-flag_name-owner_site")
     stActive26_13s = [st26_13 for st26_13 in st26_13s if re.search(r':ACTIVE', st26_13["_subTypeId"])]
-    activeFlagUse = dict((st26_13["flag_name"]["byValueCount"].keys()[0], st26_13["_total"]) for st26_13 in stActive26_13s) 
+    activeFlagUse = dict((list(st26_13["flag_name"]["byValueCount"])[0], st26_13["_total"]) for st26_13 in stActive26_13s) 
             
     mu = """---
 layout: default
@@ -124,7 +124,7 @@ def webReportFlagTypes(stationNo, activeFlagUse):
         fId = "{} [{}]".format(resource["label"], resource["_id"])
         if resource["status"] != "1:ACTIVE":
             raise Exception("Expected all to be active")
-        if len(set(resource.keys()) - (set(["_id", "type", "label", "status"]).union(set(props)))):
+        if len(set(resource) - (set(["_id", "type", "label", "status"]).union(set(props)))):
             raise Exception("Unexpected property for national flag")
         row = []
         for prop in props:
@@ -177,7 +177,7 @@ def webReportFlagTypes(stationNo, activeFlagUse):
     noAssigned = 0
     rows = []
     for resource in resourceIter:
-        if len(set(resource.keys()) - (set(["_id", "type", "label", "status"]).union(set(props)))):
+        if len(set(resource) - (set(["_id", "type", "label", "status"]).union(set(props)))):
             raise Exception("Unexpected property for national flag")
         total += 1
         if not ("status" in resource and resource["status"] == "1:ACTIVE"):
@@ -382,17 +382,16 @@ Document Tie in
                
 def main():
 
-    assert(sys.version_info >= (2,7))
+    assert sys.version_info >= (3, 4)
+
+    try:
+        stationNo = sys.argv[1]
+    except IndexError:
+        raise SystemExit("Usage _EXE_ STATIONNO")
 
-    if len(sys.argv) < 2:
-        print "need to specify station # ex/ 442 - exiting"
-        return
-        
-    stationNo = sys.argv[1]
-    
     userSiteDir = SITE_DIR_TEMPL.format(stationNo)
     if not os.path.isdir(userSiteDir):
-        raise Exception("Expect User Site to already exist with its basic contents")
+        raise SystemExit("Expect User Site to already exist with its basic contents")
     
     webReportPRFs(stationNo)
                  
