@@ -142,6 +142,8 @@ class UserClassifier:
     - .5 IEN
     - pmo, smos, keys
     - signins
+
+    BIG TODO: why no PostMaster in ALASKA/463
     
     Expect:
     - NO ssn
@@ -159,7 +161,7 @@ class UserClassifier:
         userRef = postmaster[0]
         if not re.search(r'\[200-\.5\]$', userRef):
             raise Exception("IEN not .5")
-        if userRef not in self.__st3_081sByUserRef:
+        if userRef not in self.__st3_081sByUserRef: # Made exception for 463/Alas manually but leaving here
             raise Exception("No sign ins for POSTMASTER")
         userInfo = self.__userInfoByUserRef[userRef]
         if not ("primary_menu_option" in userInfo and "secondary_menu_options" in userInfo and "keys" in userInfo):
@@ -292,8 +294,11 @@ class UserClassifier:
             raise Exception("Expected proxy user {} to have access verify".format(userRef))
                         
         if sum(1 for prop in signOnType if re.match(r'remote', prop)):
-            raise Exception("Unexpected \"remote_...\" property in SO of Proxy User {}".format(userRef))
-                        
+            # saw in BOI 531 and TERMINAL EMULATOR remote_app (may be overload use?)
+            warning = "Unexpected \"remote_...\" property in SO of Proxy User"
+            self.__result["warningsByUserRef"][userRef].append(warning)
+            print("** Warning Proxy: {} for {}".format(warning, userRef))
+                                    
         # HMP SPOK only => need to show
         if userInfo["user_class"].split(" ")[0] != "CONNECTOR":
             warning = "Only CONNECTOR Proxies but {}".format(userInfo["user_class"].split(" ")[0])
