@@ -48,10 +48,6 @@ def loadSelectTypes(stationNo):
 
 """
 Shows off key stats with a little (obvious) stats observations
-
-https://docs.scipy.org/doc/numpy/user/
-
-TODO: more on statistics (and numpy?) ... median_low etc for rounded
 """
 def reportKeyStats(kstats):
     print("Count Values: {}".format(kstats["count"]))
@@ -74,28 +70,29 @@ def reportKeyStats(kstats):
 """
 Key stats for distribution of values
 
-TODO: fully expand on statistics (median_low etc) and not just numpy repeat
-(method= ...)
+https://docs.python.org/3/library/statistics.html
 
-min, max
-Quartile: 1, 2 (Median), 3 for data-spread
+REM: values may be distributed ie/ not all responses the same size but are they
+skewed too with a few exceptions off in a left/negative or right/positive tail? 
+
+Note: ignoring sample (statistic) vs population (parameter). 
+
+Basics which work for skewed distributions (typical):
+... left (exceptions negative or left or smaller) or right where mean!=median
+... median much better measure of centeredness as isn't effected by outliers
+- min, max, mean
+- quartile 1 2 (median), 3 ie/ three cut points (quintile is five)
+  ... Medieval Latin quartilis from Classical Latin quartus, fourth
 ... "five-number summary" ideal for data w/ outliers and a 'box-and-whisker' plot 
 ... Quartiles divide a rank-ordered data set into four equal parts at 25, 50, 75
 percentiles
-IQR
-... "midspread" - best at spread if outliers
-olt < (Q1-1.5*IQR)
-oht > Q3+1.5*IQR
-... for key outliers: < are lows, > are highs. Only present if > max, < min
-mean
-std
-... for symm distributions with no outliers where five numbers don't kick in
+- IQR ... "midspread", inter-quartile range
+- olt < (Q1-1.5*IQR) ... 1.5 == "inner fences"
+- oht > Q3+1.5*IQR ... 3 == "outer fences"
 
-TODO: consider <span class='yellowIt'>{:.2f}%</span> and numpy.around or
-roundFloat (see below)
-
-oht: 1.5 == "inner fences"
-major outliers: Q3 + 3*IQR == "outer fences" ... need name for this.
+For not skewed (mean = median)
+- standard deviation (using population form, not n-1 sample form)
+- co-efficient of variation
 
 Note: this is 'Univariate method' (See: https://www.kdnuggets.com/2017/01/3-methods-deal-outliers.html / 'Minkowski error' etc)
 
@@ -106,6 +103,7 @@ ie/ K times Quartile. Fences for box plots.
 """    
 def keyStats(values):
     try: # tmp allow for < 3.8
+        # argument n=4 default => quartile from quantile method
         quartiles = statistics.quantiles(values, method="inclusive") # inc matched numpy
     except:
         import numpy
@@ -122,7 +120,11 @@ def keyStats(values):
     cntValsGTOHTO = sum(1 for val in values if val > ohto) 
     mx = sorted(values)[-1] # numpy.max(values)
     mn = sorted(values)[0] # numpy.min(values)
-    stats = {"total": sum(val for val in values), "count": len(values), "min": mn, "quartileOne": quartileOne, "median": median, "quartileThree": quartileThree, "max": mx, "iqr": iqr, "oht": oht, "ohto": ohto, "mean": statistics.mean(values), "std": statistics.stdev(values)}
+    meen = statistics.mean(values)
+    pstd = statistics.pstdev(values) # for Population
+    cvar = pstd/meen # co-efficient of variation
+    stats = {"total": sum(val for val in values), "count": len(values), "min": mn, "quartileOne": quartileOne, "median": median, "quartileThree": quartileThree, "max": mx, "iqr": iqr, "oht": oht, "ohto": ohto, "mean": meen, "pstd": pstd, "cvar": cvar}
+
     olt = quartileOne - (1.5 * iqr) # Outlier Low below this
     if int(olt) > 0: # only olt if > 0 ie/ negative ie/ all acceptable
         stats["olt"] = olt
@@ -157,36 +159,6 @@ def flattenFrequencyDistribution(freqDistrib, forceInt=True):
         freqDistribFlat.extend(f)
     return freqDistribFlat
     
-"""
-New Stats to expand:
-
-- (basic) and relative (percent/ration) frequency distribution ie/ cnt occurrences
-        from collections import Counter
-        Counter([...]) ... 
-  ... put relatives into a stacked bar chart
-  ... conditional => a number of freq distribs, one each per condition (ex/ for PDFs, WSs)
-- bar (or a pie) to histogram: when # of distict values too big then go to histogram
-- null hypothesis/expected frequencies: variables unrelated (type of image and WS?)
-  - https://www.spss-tutorials.com/chi-square-independence-test/
-"""
-
-"""
-- cumulative frequencies add as go (https://www.spss-tutorials.com/frequency-distribution-what-is-it/) ... no good for nominals ("French" or "PDF") but for good/better/best ...
-  ie age of patients  ... https://www.wyzant.com/resources/lessons/math/statistics_and_probability/averages/cumulative_frequency_percentiles_and_quartiles
-  ... A cumulative frequency graph, also known as an Ogive
-  ... good site for stuff.
-  ... cumulative freq can be used to gen quartiles
-"""        
-
-"""
-Clustering, numpy has it (ntlk uses it - but mainly see intelligent web book)
-... k ... and clumping 'users' or ... based on similarity/distance.
-"""
-
-"""
-after excel, try matplotlib, networkx 
-"""
-    
 # ############################ Simple Report Utils #################
 
 """
@@ -251,7 +223,7 @@ def reduce200(stationNo, onlyUsers=None, forceNew=False, coreOnly=False):
         {"fileType": "200", "check": "ALL"} # Index
     ])
     if not allThere:
-        raise Exception("Some required data is missing - {}".format(details))
+        raise Exception("Some required data of {} is missing - {}".format(stationNo, details))
 
     print("First time through, creating index by id for Users (200) for {}".format(stationNo))
     
@@ -672,6 +644,12 @@ def reduce4(stationNo):
                 if ref["id"] == resource["_id"]:
                     continue # avoid self reference in Puget being parent of Puget
                 red[apred] = ref
+                if apred == "VISN":
+                    try:
+                        visnNo = int(ref["label"].split(" ")[1])
+                    except:
+                        raise Exception(f'Institution {red["ien"]}/{red["label"]} has non numeric VISN value {ref["label"].split(" ")[1]}')
+                    res["visn"] = visnNo   
         # EX/ VISTANUM == stationNumber but npi too (also explicit?)
         # ... has status and effective_date_time too
         # ... https://npiregistry.cms.hhs.gov/registry/provider-view/1477502946 ... 
@@ -685,7 +663,7 @@ def reduce4(stationNo):
         # taxonomy_code - reduce to list
     json.dump(infosByIEN, open(jsnFl, "w"), indent=4)
     return infosByIEN
-
+    
 """
 Utility for making tmp's - flatten out dates, pters etc
 
@@ -708,6 +686,154 @@ def flattenPropValues(resource, nixTypeLabel=True): # not recursive
             continue
         if "id" in resource[prop]:
             resource[prop] = "{} [{}]".format(resource[prop]["label"], resource[prop]["id"])
+            
+# ##################### Generate/Use VISN/Station Institution Map ###################
+
+from .visnNVistAs import byVISNByVistAStation
+
+"""
+To use:
+   > vistasOfVISNByOne("668")
+"""
+def vistasOfVISNByOne(oneVistASNO):
+    return vistasOfVISN(visnOfVistA(oneVistASNO))
+
+"""
+To use:
+   > vistasOfVISN("20") -- all VISN 20 VistAs as {"{SNO}": "{MN}" ...}
+"""
+def vistasOfVISN(onlyVISN=""):
+    mainsByVISN = defaultdict(dict)
+    for visn in sorted(byVISNByVistAStation, key=lambda x: int(x)):
+        for sno in byVISNByVistAStation[visn]:
+            mainInstance = byVISNByVistAStation[visn][sno][0]
+            if "mn" not in mainInstance:
+                continue
+            mainsByVISN[visn][sno] = mainInstance["mn"]
+    if onlyVISN == "":
+        return mainsByVISN
+    return mainsByVISN[onlyVISN]
+
+"""
+To use:
+   > visnOfVistA("668") -- get "20"
+"""
+def visnOfVistA(stationNo):
+    for visn in byVISNByVistAStation:
+        if stationNo in byVISNByVistAStation[visn]:
+            return visn
+    return ""
+
+"""
+Different from reduce4, this produces the VISN/VistA-based Station Map which goes into
+fmqlreports itself at the top level.
+
+Default saved to tmp of specific vista but main use is to populate/refresh the visn/vista
+definition of visnNVistAs.py in fmqlreports.
+
+20: {
+    668: [
+        { spokane
+        { coeur ...
+}
+
+Revisit: may remove veterans homes etc (don't appear to have NPIs)
+
+QA with https://www.va.gov/find-locations/facility/vha_531GJ (etc)
+
+TODO: 
+- add in geo data based on addresses and urls and reproduce something like
+(but better than) https://www.va.gov/directory/guide/region.asp?ID=1010 
+- find how to add mn's to agreed values (ex/ WCO not WCI for White City). Not in 4? so fixing for VISN 20 but beyond?
+"""
+def reduce4VISNVistASNO(stationNo, saveTo=""):
+
+    # Pass 1: take relevant institutions (remaining outliers deleted in phase 2)
+    dataLocn = f'{VISTA_DATA_BASE_DIR}{stationNo}/Data'
+    resourceIter = FilteredResultIterator(dataLocn, "4") 
+    stationNosSeen = set()
+    instits = []
+    for resource in resourceIter:
+        if not (
+            "status" in resource and 
+            resource["status"] == "N:National"
+        ):
+            continue
+        if "inactive_facility_flag" in resource:
+            continue
+        if "station_number" not in resource:
+            continue
+        if "associations" not in resource:
+            continue
+        # There are some CEMETERY before but its a mix
+        if int(resource["station_number"][0:3]) >= 780:
+            continue 
+        if int(resource["station_number"][0:3]) < 400:
+            continue 
+        # Only do top level with locked IENs (removing as 463 in VISN 20 687 is 363)
+        # if len(resource["station_number"]) == 3 and resource["station_number"] != resource["_id"].split("-")[1]:
+        if resource["station_number"] in stationNosSeen:
+            raise Exception("Station No reused")
+        stationNosSeen.add(resource["station_number"])
+        instit = {"name": resource["name"], "stationNumber": resource["station_number"]}
+        if "official_va_name" in resource and resource["official_va_name"] != resource["name"]:
+            instit["officialName"] = resource["official_va_name"]
+        if "npi" in resource:
+            instit["npi"] = resource["npi"]
+        if "domain" in resource:
+            instit["domain"] = resource["domain"]["label"]     
+        visnAssocs = [assoc for assoc in resource["associations"] if "associations" in assoc and assoc["associations"]["label"] == "VISN" and "parent_of_association" in assoc and assoc["parent_of_association"]["label"].split(" ")[0] == "VISN"]
+        if len(visnAssocs) > 1:
+            raise Exception("Ambiguous VISN declaration")
+        if len(visnAssocs): # missing filled in below
+            instit["visn"] = visnAssocs[0]["parent_of_association"]["label"].split(" ")[1]
+        instits.append(instit)
+        
+    # Pass 2: 3 purposes - [1] nix entries with 'bad' stations ex/ 452XX that don't
+    # fit into any VistA, [2] puts main center of a VistA first AND [3] gives normalized
+    # artificial IEN to entries ie/ global IENs
+    stationGroups = dict((instit["stationNumber"], [instit]) for instit in instits if len(instit["stationNumber"]) == 3)
+    visnStationGroups = defaultdict(dict)
+    MNFIXED = {"463": "ANC", "531": "BOI", "687": "WWW", "668": "SPO", "648": "POR", "653": "ROS", "663": "PUG", "692": "WCO", "757": "COS"} # 20 + Columbus
+    for sgno in stationGroups:
+        grpVISNs = set(instit["visn"] for instit in stationGroups[sgno] if "visn" in instit)
+        if len(grpVISNs) != 1:
+            raise Exception("A Vista/Station Group has no VISN or > 1 VISN")
+        mainInstit = stationGroups[sgno][0]
+        mainInstit["ien"] = sgno # ie/ 668 IEN is 668
+        if "visn" not in mainInstit:
+            raise Exception("Expect all main hospitals of VISN to spec VISN")
+        visnStationGroups[mainInstit["visn"]][sgno] = [mainInstit]
+        if sgno in MNFIXED:
+            mainInstit["mn"] = MNFIXED[sgno]
+            continue
+        if "domain" in mainInstit:
+            mainInstit["mn"] = re.sub(r'\-', '', mainInstit["domain"])[0:3]
+    for instit in sorted(instits, key=lambda x: x["stationNumber"]):
+        if len(instit["stationNumber"]) == 3:
+            continue
+        sgno = instit["stationNumber"][0:3]
+        if sgno not in stationGroups: # Purge extras - not for VistA
+            continue
+        stationGroups[sgno].append(instit)
+        # Ex/ 687HA would be 68701, 68702 ... ie/ up to 99 places and 687
+        # ie/ three long is for main center as it is now.
+        instit["ien"] = int("{}{}".format(sgno, str(len(stationGroups[sgno]) - 1).zfill(2)))         
+        if "visn" not in instit:
+            instit["visn"] = stationGroups[sgno][0]["visn"] 
+        visnStationGroups[instit["visn"]][sgno].append(instit)  
+        
+    red = {
+        "about": {
+            "generated": datetime.now().strftime("%Y-%m-%d"), 
+            "fromVistA": stationNo, 
+            "fromFile": 4
+        },
+        "byVISNByVistAStation": visnStationGroups
+    }
+    jsnFl = saveTo if saveTo else TMPWORKING_LOCN_TEMPL.format(stationNo) + "4VISNVistASNOReduction.json"
+    json.dump(red, open(jsnFl, "w"), indent=4)
+    return red 
     
 # ############################ Simple Reusable MD Makers ###############################
 
@@ -753,7 +879,7 @@ def makePlots(stationNo, plotDataFileSuffix):
     import pandas as pd
 
     from visualize import Visualize
-    imageDir = f'{VISTA_DATA_BASE_DIR}{stationNo}/ReportSite/Images
+    imageDir = f'{VISTA_DATA_BASE_DIR}{stationNo}/ReportSite/Images'
     if not os.path.isdir(imageDir):
         os.mkdir(imageDir)
     viz = Visualize(imageDir)
